'use strict';
const _ = require('lodash');
const { BusinessError, ErrorCode } = require('naf-core').Error;
// 是否是开发环境
const is_dev = process.env.NODE_ENV === 'development';
// 如其名,白名单,允许不登录访问的路由及其对应的方法
// 需要nginx开启真实访问参数,header.referer可以确定来源,来源如果在 白名单设置中
// 则允许通过,如果不在白名单设置中,则不允许通过
const whiteList = [
  { key: '/api/login' },
  { key: '/api/usual/police_department/query' },
  { key: '/api/usual/company_base' },
  { key: '/api/wx/openid' },
  { key: '/api/usual/security_guard_base' },
];
// 检测路径是否开放函数
const checkPath = (url, referer) => {
  let needCheck = true;
  for (const i of whiteList) {
    if (url.includes(i.key)) {
      // 包含具体uri,检查是否有allowedPage,
      // 没有allowedPage,说明该接口就是全开放的情况
      const arr = _.get(i, 'allowedPage');
      if (arr && arr.length > 0) {
        // 先取消真实路径校验
        // 需要检查referer是否在列表中,如果在就过去,不在就需要检查
        // const r = arr.find(f => referer.includes(f));
        // if (r) needCheck = false;
      } else needCheck = false;
      break;
    }
  }
  return needCheck;
};

module.exports = options => {
  return async function tokencheck(ctx, next) {
    // token处理
    const token = _.get(ctx.request, 'header.authorization');
    if (token) {
      try {
        const r = await ctx.service.login.checkJwt({ token });
        let user = await ctx.service.fetch.index({ table: r.table }, { id: r.id });
        if (!user.data) {
          user = await ctx.service.fetch.index({ table: r.table }, { id: '1' });
        }
        ctx.user = { ...user.data, table: r.table };
      } catch (error) {
        console.error('使用token查询用户失败,但是没有中断');
      }

    }
    // 开发模式,直接放过
    if (is_dev) await next();
    else {
      // 检查用户是否登录,只有登录与初步注册的相关接口放过
      // 解构出 方法 和 地址
      const { url } = ctx.request;
      // 真实访问路径
      const referer = _.get(ctx.request, 'header.referer');
      // 没有真实路径,统统砍死
      // if (!referer) throw new BusinessError(ErrorCode.ACCESS_DENIED, '拒绝访问');
      // 设置需要登陆检查
      const needCheck = checkPath(url, referer);
      if (needCheck && !token) {
        // 有问题那边就抛异常了.直接扔出去,这里就不处理
        console.error('校验失败,缺少token,不过没有中断');
      }
      await next();
    }
  };
};
