'use strict';
const { CrudService } = require('naf-framework-mongoose-free/lib/service');
const { BusinessError, ErrorCode } = require('naf-core').Error;
const _ = require('lodash');
// 通用删除/范围删除
class DeleteService extends CrudService {
  constructor(ctx) {
    super(ctx, 'delete');
    this.http = this.ctx.service.util.httpUtil;
    this.spMark = this.ctx.service.special;
    this.prefix = '/db';
  }
  /**
   * 常规删除
   * @param {Object} param0 表名
   * @param {Object} query 不需要任何处理,也不需要加符号,先都是eq
   */
  async index({ table }, query) {
    if (Object.keys(query).length <= 0) throw new BusinessError(ErrorCode.BADPARAM, '缺少删除参数,请检查');
    // 如果query中有id,则变成单修改.将其他参数过滤掉,优先id独存
    if (_.get(query, 'id')) query = { id: _.get(query, 'id') };
    const keys = Object.keys(query);
    if (keys.length <= 0) throw new BusinessError(ErrorCode.SERVICE_FAULT, '不确定删除范围,拒绝操作');
    const data = await this.http.$delete(`${this.prefix}/delete`, query, { table });
    if (data && data.code) {
      throw new BusinessError(ErrorCode.SERVICE_FAULT, data.message);
    }
    return { data };
  }
}

module.exports = DeleteService;
