'use strict';
const { CrudService } = require('naf-framework-mongoose-free/lib/service');
const { BusinessError, ErrorCode } = require('naf-core').Error;
const _ = require('lodash');
const assert = require('assert');

// 保安员入职相关操作
class WorkService extends CrudService {
  constructor(ctx) {
    super(ctx, 'work');
    this.createService = this.ctx.service.create;
    this.updateService = this.ctx.service.update;
    this.deleteService = this.ctx.service.delete;
  }

  /**
   * 批量办理入职的实现(整理数据后才能执行)
   * @param {Array} data 数据[object]
   */
  async toWork({ data }) {
    if (_.isArray(data) && data.length <= 0) {
      throw new BusinessError(ErrorCode.DATA_INVALID, '数据错误');
    }
    const errorList = [];
    // 创建入职信息
    for (const d of data) {
      let resultData;
      try {
        const res = await this.createService.index({ table: 'company_baoan_work' }, d);
        if (res.data) resultData = res.data;
      } catch (error) {
        errorList.push({ message: `${d.name}:创建入职信息失败` });
        continue;
      }
      if (resultData) {
        try {
          // 创建入职信息,随后将保安证的企业与企业id修改
          const body = _.pick(d, [ 'company_id', 'company_name' ]);
          await this.updateService.index({ table: 'certificates_base' }, { card: d.card }, body);
        } catch (error) {
          errorList.push({ message: `${d.name}:修改保安证 失败!` });
          await this.deleteService.index({ table: 'company_baoan_work' }, { id: resultData.id });
        }
      }
    }

    return errorList;
  }
}

module.exports = WorkService;
