'use strict';
const _ = require('lodash');
const Service = require('egg').Service;
const { BusinessError, ErrorCode } = require('naf-core').Error;
// 数据进库
const dataToDb = require('../public/dataToDb');
// 数据出库
const dbToData = require('../public/dbToData');
// 特殊符号处理
class SpecialService extends Service {
  constructor(ctx) {
    super(ctx, 'special');
    // 特殊符号列表
    this.list = [ '@', '^' ];
    // in的分隔符
    this.inSplit = ',';
  }

  /**
   * query的特殊符号处理
   * 有新的符号,就在else那里处理
   * @param {Object} obj 参数对象
   */
  dealMarkQuery(obj) {
    // 处理严谨点,要是真有人改了代码,不传参就调用别在这报错,工具函数,牵一发动全身,危险
    const mark = this.hasMark(obj.key);
    if (obj && obj.key && mark) {
      const key = obj.key;
      let sp = [];
      sp = key.split(mark);
      if (mark === '@') {
        // ${key}@start/end 形式
        // 单独处理下value,不一定都是日期格式,所以不是日期直接给过去,是日期就转换;
        obj.key = _.head(sp) || key;
        if (_.last(sp) === 'start') {
          obj.type = 'gte';
        } else if (_.last(sp) === 'end') {
          obj.type = 'lte';
        }
      } else if (mark === '^') {
        // ^${key}的形式; value 统一用','号分隔
        // 转换成 in 的 type
        obj.type = 'in';
        obj.key = _.last(sp);
      } else new BusinessError(ErrorCode.DATA_INVALID, '未知特殊符号,请联系管理员通知开发人员');
    }
    return obj;
  }

  /**
   * fetch特殊符号处理
   * @param {Object} obj 请求参数,和query的处理不一样,这个obj没经过任何处理,是直接从前端传来的 Object类型
   */
  dealMarkFetch(obj) {
    for (const key in obj) {
      // String.split(reg/string)是分割,不是扔了,没有分割的话起码有个原来的值,别干了啊,干了就没了
      let sp = [];
      if (key.includes('@')) {
        // 时间需要改值
        sp = key.split('@');
        obj[_.head(sp)] = obj[key];
      }
      // 分割出来了,原来的删了,上面增加完了
      if (sp.length > 1) delete obj[key];
    }
    return obj;
  }

  /**
   * 查询 键名 是否带 特殊符号,如果带,返回对应的 特殊符号;不带,返回false
   * @param {String} key 带特殊符号的键名
   */
  hasMark(key) {
    let result = false;
    for (const mark of this.list) {
      if (key.includes(mark)) {
        result = mark;
        break;
      } else continue;
    }
    return result;
  }

  /**
   * 转换日期格式,进oracle能查询--吐了,真吐了
   * @param {String} str 日期字符串
   */
  changeDate(str) {
    return dataToDb.toUnix(str);
  }

  /**
   * 处理是否需要转换字段的值, 按规定处理 出库 数据
   * @param {Object/Array} object 查询后的结果
   * @param {String} table 表名
   */
  resetDataToFront(object, table) {
    const method = _.get(dbToData, table);
    if (!method) return object;
    if (_.isArray(object)) {
      // 数组,逐条处理
      object = object.map(i => {
        i = method(i);
        return i;
      });
    } else if (_.isObject(object)) {
      // 对象,单对象处理
      object = method(object);
    }
    return object;
  }

  /**
   * 处理是否需要转换字段的值, 按规定处理 入库 数据
   * @param {Object/Array} object 入库的数据
   * @param {String} table 表名
   */
  resetDataToDB(object, table) {
    const method = _.get(dataToDb, table);
    if (!method) return object;
    if (_.isArray(object)) {
      // 数组,逐条处理
      object = object.map(i => {
        i = method(i);
        return i;
      });
    } else if (_.isObject(object)) {
      // 对象,单对象处理
      object = method(object);
    }
    return object;
  }
}

module.exports = SpecialService;
