'use strict';
const { CrudService } = require('naf-framework-mongoose-free/lib/service');
const { BusinessError, ErrorCode } = require('naf-core').Error;
const _ = require('lodash');
// 通用修改/范围修改
class UpdateService extends CrudService {
  constructor(ctx) {
    super(ctx, 'update');
    this.http = this.ctx.service.util.httpUtil;
    this.spMark = this.ctx.service.special;
    this.prefix = '/db';
  }
  /**
   * 常规修改
   * @param {Object} param0 表名
   * @param {Object} query 不需要任何处理,也不需要加符号,先都是eq
   * @param {Object} body 添加数据
   */
  async index({ table }, query, body) {
    body = this.dealBody(table, body);
    // 如果query中有id,则变成单修改.将其他参数过滤掉,优先id独存
    if (_.get(query, 'id')) query = { id: _.get(query, 'id') };
    const keys = Object.keys(query);
    if (keys.length <= 0) throw new BusinessError(ErrorCode.SERVICE_FAULT, '不确定修改范围,拒绝操作');
    const data = await this.http.$post(`${this.prefix}/update`, { data: body, query }, { table });
    if (data && data.code) {
      throw new BusinessError(ErrorCode.SERVICE_FAULT, data.message);
    }
    return { data };
  }

  /**
   * 额外处理入库数据
   * @param {String} table 表名
   * @param {Object} body 数据
   */
  dealBody(table, body) {
    if (Object.keys(body).length <= 0) throw new BusinessError(ErrorCode.BADPARAM, '缺少数据,请检查请求');
    // 检查参数处理
    body = this.spMark.resetDataToDB(body, table);

    return body;
  }
}

module.exports = UpdateService;
